MODULE DaysToDate;
IMPORT IO, ConSigs;

TYPE
  TDaysInMonth = ARRAY [0..12] OF CARDINAL;

CONST
  DaysInMonth = TDaysInMonth{ 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365 };

(*
PROCEDURE DaysInMonth( m: CARDINAL; l: BOOLEAN ): CARDINAL;
BEGIN
  RETURN daysInMonth[ m ] + ORD( l ) * ORD( m > 1 );
END DaysInMonth;
*)

(* Prft, ob ein Jahr ein Schaltjahr ist *)
PROCEDURE IsLeapYear( year: LONGINT ): BOOLEAN;
BEGIN
  RETURN ( year MOD 4 = 0 ) & ( year MOD 100 # 0 ) OR ( year MOD 400 = 0 ) ;
END IsLeapYear;

(* Berechnet das Datum aus der Anzahl der Tage seit 1.1.0 *)
PROCEDURE DaysToDate( days: LONGINT; VAR year: LONGINT; VAR month, day: CARDINAL );
CONST
  p400 = 365 * 400 + 97;  (* 146097 *)
  p100 = 365 * 100 + 24;  (*  36524 *)
  p4   = 365 *   4 +  1;  (*   1461 *)
  p1   = 365 *   1 +  0;  (*    365 *)
VAR
  y400, y100, y4, y1: LONGINT;
  l: BOOLEAN;
  i: CARDINAL;
BEGIN
  (* 1. Berechnung der Jahre *)
  (* 1a: 400-Jahres-Zyklen *)
  y400 := days DIV p400;
  DEC( days, y400 * p400 );  (* days = days MOD p400, 0 <= days < p400 *)

  (* 1b: 100-Jahres-(Pseudo-)Zyklen *)
  y100 := ( days - 1 ) DIV p100;
  IF y100 <= 0 THEN
    y100 := 0;
    (* all days remain, 0 <= days <= p100 *)
    y4 := days DIV p4;
    DEC( days, y4 * p4 );  (* days = days MOD p4, 0 <= days < p4 *)
    y1 := ( days - 1 ) DIV p1;
    IF y1 <= 0 THEN
      y1 := 0;
      (* all days remain, 0 <= days <= p1 *)
    ELSE (* y1 IN {1,2,3} *)
      DEC( days, 1 + y1 * p1 );  (* days = days MOD p1, 0 <= days < p1 *)
    END;
  ELSE (* y100 IN {1,2,3} *)
    DEC( days, 1 + y100 * p100 );  (* days = days MOD p100, 0 <= days < p100 *)
    y4 := ( days + 1 ) DIV p4;
    IF y4 = 0 THEN
      (* all days remain, 0 <= days < p4 *)
      y1 := days DIV p1;
      DEC( days, y1 * p1 );  (* days = days MOD p1, 0 <= days < p1  *)
    ELSE (* y4 IN {1..25} *)
      DEC( days, y4 * p4 - 1 );  (* days = days MOD p4, 0 <= days < p4 *)
      y1 := ( days - 1 ) DIV p1;
      IF y1 <= 0 THEN
        y1 := 0;
        (* all days remain, 0 <= days <= p1 *)
      ELSE (* y1 IN {1,2,3} *)
        DEC( days, 1 + y1 * p1 );  (* days = days MOD p1, 0 <= days < p1 *)
      END;
    END;
  END;
  year := y400 * 400 + y100 * 100 + y4 * 4 + y1;
  l := IsLeapYear( year );

  (* 2. Berechnung des Monats *)
  month := 1;
  i := 1;
  WHILE i <= 12 DO
    IF CARDINAL( days ) < DaysInMonth[ i ] + ORD( l ) * ORD( i > 1 ) THEN
      month := i;
      i := 13; (* Beende die Schleife *)
    ELSE
      INC(i);
    END;
  END;

  (* 3. Berechnung des Tages *)
  day := CARDINAL( days ) - ( DaysInMonth[ month - 1 ] + ORD( l ) * ORD( month > 2 ) ) + 1;
END DaysToDate;

VAR
  days, y: LONGINT;
  m, d: CARDINAL;
BEGIN
  IO.WrStr( "DaysToDate v1, 2025-02-06, CTRL-C to exit" ); IO.WrLn();

  LOOP
    (* Eingabe: Anzahl der Tage seit 1.1.0 *)
    IO.WrStr( "Enter days since 0000-01-01: " );
    days := IO.RdLngInt();

    (* Berechnung des Datums *)
    DaysToDate( days, y, m, d );

    (* Ausgabe des Ergebnisses *)
    IO.WrStr( "Date: " );
    IF y < 0 THEN IO.WrChar( '-' ); y := -y END;
    IO.PrefixChar := '0';
    IO.WrLngInt( y, 4 ); IO.WrStr( "/" );
    IO.WrCard( m, 2 ); IO.WrStr( "/" );
    IO.WrCard( d, 2 ); IO.WrLn();
  END;
END DaysToDate.
